Package: easycensus
Title: Quickly Find, Extract, and Marginalize U.S. Census Tables
Version: 1.1.4
Authors@R: 
    person("Cory", "McCartan", email="mccartan@psu.edu", role=c("aut", "cre"))
Description: Extracting desired data using the proper Census variable names can 
    be time-consuming. This package takes the pain out of that process by 
    providing functions to quickly locate variables and download labeled tables 
    from the Census APIs (<https://www.census.gov/data/developers/data-sets.html>).
Depends: R (>= 2.10)
Imports: rlang, vctrs, pillar, dplyr (>= 1.0.0), tidyr (>= 1.0.0),
        stringr, censusapi, cli
Suggests: posterior, testthat (>= 3.0.0)
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
URL: https://corymccartan.com/easycensus/,
        https://github.com/CoryMcCartan/easycensus/,
        http://corymccartan.com/easycensus/
BugReports: https://github.com/CoryMcCartan/easycensus/issues
Config/testthat/edition: 3
Language: en-US
NeedsCompilation: no
Packaged: 2026-05-13 17:09:30 UTC; cmccartan
Author: Cory McCartan [aut, cre]
Maintainer: Cory McCartan <mccartan@psu.edu>
Repository: CRAN
Date/Publication: 2026-05-14 05:10:14 UTC
Built: R 4.5.2; ; 2026-05-14 09:46:52 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(format,estimate)
S3method(pillar_shaft,estimate)
S3method(print,cens_table)
S3method(vec_arith,estimate)
S3method(vec_arith.estimate,MISSING)
S3method(vec_arith.estimate,default)
S3method(vec_arith.estimate,estimate)
S3method(vec_arith.estimate,numeric)
S3method(vec_arith.numeric,estimate)
S3method(vec_cast,double.estimate)
S3method(vec_cast,estimate.double)
S3method(vec_cast,estimate.estimate)
S3method(vec_cast,estimate.integer)
S3method(vec_math,estimate)
S3method(vec_proxy_compare,estimate)
S3method(vec_ptype2,double.estimate)
S3method(vec_ptype2,estimate.double)
S3method(vec_ptype2,estimate.estimate)
S3method(vec_ptype2,estimate.integer)
S3method(vec_ptype2,integer.estimate)
S3method(vec_ptype_abbr,estimate)
export(as_estimate)
export(cens_find)
export(cens_find_acs)
export(cens_find_dec)
export(cens_geo)
export(cens_get_acs)
export(cens_get_dec)
export(cens_get_raw)
export(cens_margin_to)
export(cens_parse_tables)
export(est_pct_chg)
export(est_prop)
export(estimate)
export(get_est)
export(get_moe)
export(get_se)
export(is_estimate)
export(tidy_age)
export(tidy_age_bins)
export(tidy_ethnicity)
export(tidy_income_bins)
export(tidy_parens)
export(tidy_race)
export(tidy_race_detailed)
export(tidy_simplify)
export(to_rvar)
import(cli)
import(stringr)
import(vctrs)
importFrom(dplyr,"%>%")
importFrom(dplyr,as_tibble)
importFrom(dplyr,if_else)
importFrom(pillar,pillar_shaft)
importFrom(rlang,.data)
importFrom(rlang,.env)
importFrom(stats,qnorm)
importFrom(stats,rnorm)
importFrom(utils,adist)
importFrom(utils,data)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# **easycensus** <a href="https://corymccartan.com/easycensus/"><img src="man/figures/logo.png" align="right" height="138" /></a>

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/easycensus)](https://CRAN.R-project.org/package=easycensus)
[![License:
MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/license/mit)
[![R-CMD-check](https://github.com/CoryMcCartan/easycensus/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/CoryMcCartan/easycensus/actions/workflows/R-CMD-check.yaml)
[![Lifecycle:
stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
<!-- badges: end -->

Extracting desired data using the proper Census variable names can be
time-consuming. This package takes the pain out of that process.

The use case is best illustrated by an example. Suppose you want
age-by-race information at the tract level. Unfortunately, the Census
Bureau doesn’t publish a specific age-by-race table. You could build one
yourself from public-use microdata, but that lacks tract-level
geographic information, for privacy reasons. So you are left trying to
find an existing Census product that you can extract age-by-race
information from.

Unless you’re a Census pro, you won’t know what exactly what is off the
top of your head. But suppose you know you’d like to get the data from
the decennial census, since it covers the whole nation and asks about
age and race. `easycensus` provides the `cens_find_dec()` function to
help you locate exactly which decennial census table to use to get the
data you want.

``` r
library(easycensus)

cens_find_dec(age, race)
#> 
#> ── Top 2 matching tables ───────────────────────────────────────────────────────
#> 
#>  P12  - SEX BY AGE
#> Surveys / Files:
#> ✔ Decennial / Summary File 1
#> Parsed variables:
#> • sex
#> • age
#> • race_ethnicity
#> Example values:
#> • female / 45 to 49 years / black or african american alone
#> • female / 20 years / two or more races
#> • male / 55 to 59 years / american indian and alaska native alone
#> 
#>  PCT12  - SEX BY AGE
#> Surveys / Files:
#> ✔ Decennial / Summary File 1
#> Parsed variables:
#> • sex
#> • age
#> • race_ethnicity
#> Example values:
#> • female / 70 years / total
#> • female / 85 years / native hawaiian and other pacific islander alone
#> • female / 30 years / two or more races
```

We can see right away that our best bet is either table `P12` or table
`PCT12`, depending on whether we want age in 5-year groups or down to
individual years. Let’s say you’re OK with the five-year bins. Then all
you need to do to get your data is to call `cens_get_dec()`.

``` r
d_cens = cens_get_dec("P12", "tract", state="AK", county="Nome")
print(d_cens)
#> # A tibble: 960 × 7
#>    GEOID       NAME                    variable value sex   age   race_ethnicity
#>    <chr>       <chr>                   <chr>    <dbl> <fct> <fct> <fct>         
#>  1 02180000100 Census Tract 1, Nome C… P012002   3053 male  total total         
#>  2 02180000100 Census Tract 1, Nome C… P012003    359 male  unde… total         
#>  3 02180000100 Census Tract 1, Nome C… P012004    318 male  5 to… total         
#>  4 02180000100 Census Tract 1, Nome C… P012005    294 male  10 t… total         
#>  5 02180000100 Census Tract 1, Nome C… P012006    165 male  15 t… total         
#>  6 02180000100 Census Tract 1, Nome C… P012007    130 male  18 a… total         
#>  7 02180000100 Census Tract 1, Nome C… P012008     53 male  20 y… total         
#>  8 02180000100 Census Tract 1, Nome C… P012009     47 male  21 y… total         
#>  9 02180000100 Census Tract 1, Nome C… P012010    150 male  22 t… total         
#> 10 02180000100 Census Tract 1, Nome C… P012011    204 male  25 t… total         
#> # ℹ 950 more rows
```

Once you’ve gotten your labeled data, it’s easy to marginalize out the
unneeded `sex` variable. You can either use `group_by()` and
`summarize()` as usual, or you can use the `cens_margin_to()` function
in `easycensus`. This has the added advantage of automatically handling
margins of error for ACS data.

``` r
library(dplyr)

d_cens = d_cens %>%
    # Drop table margins. Can also use `drop_total=TRUE` in `get_dec_table()`
    filter(age != "total", race_ethnicity != "total") %>%
    cens_margin_to(age, race=race_ethnicity)
print(d_cens)
#> # A tibble: 414 × 5
#>    GEOID       NAME                                     age          race  value
#>    <chr>       <chr>                                    <fct>        <fct> <dbl>
#>  1 02180000100 Census Tract 1, Nome Census Area, Alaska 10 to 14 ye… amer…   524
#>  2 02180000100 Census Tract 1, Nome Census Area, Alaska 10 to 14 ye… asia…     1
#>  3 02180000100 Census Tract 1, Nome Census Area, Alaska 10 to 14 ye… blac…     1
#>  4 02180000100 Census Tract 1, Nome Census Area, Alaska 10 to 14 ye… hisp…     3
#>  5 02180000100 Census Tract 1, Nome Census Area, Alaska 10 to 14 ye… nati…     0
#>  6 02180000100 Census Tract 1, Nome Census Area, Alaska 10 to 14 ye… some…     0
#>  7 02180000100 Census Tract 1, Nome Census Area, Alaska 10 to 14 ye… two …    23
#>  8 02180000100 Census Tract 1, Nome Census Area, Alaska 10 to 14 ye… whit…    11
#>  9 02180000100 Census Tract 1, Nome Census Area, Alaska 10 to 14 ye… whit…    10
#> 10 02180000100 Census Tract 1, Nome Census Area, Alaska 15 to 17 ye… amer…   293
#> # ℹ 404 more rows
```

Finally, you might want to simplify the age and race labels, since they
are kind of verbose. `easycensus` provides a set of `tidy_*()` functions
to assist with this.

``` r
d_cens %>%
    mutate(race = tidy_race(race),
           tidy_age_bins(age))
#> # A tibble: 414 × 7
#>    GEOID       NAME                            age   race  value age_from age_to
#>    <chr>       <chr>                           <fct> <fct> <dbl>    <dbl>  <dbl>
#>  1 02180000100 Census Tract 1, Nome Census Ar… 10 t… aian    524       10     14
#>  2 02180000100 Census Tract 1, Nome Census Ar… 10 t… asian     1       10     14
#>  3 02180000100 Census Tract 1, Nome Census Ar… 10 t… black     1       10     14
#>  4 02180000100 Census Tract 1, Nome Census Ar… 10 t… hisp      3       10     14
#>  5 02180000100 Census Tract 1, Nome Census Ar… 10 t… nhpi      0       10     14
#>  6 02180000100 Census Tract 1, Nome Census Ar… 10 t… other     0       10     14
#>  7 02180000100 Census Tract 1, Nome Census Ar… 10 t… two      23       10     14
#>  8 02180000100 Census Tract 1, Nome Census Ar… 10 t… white    11       10     14
#>  9 02180000100 Census Tract 1, Nome Census Ar… 10 t… whit…    10       10     14
#> 10 02180000100 Census Tract 1, Nome Census Ar… 15 t… aian    293       15     17
#> # ℹ 404 more rows
```

Dive into the
[reference](https://corymccartan.com/easycensus/reference/) to learn
more!

## Installation

You can install the released version of easycensus from
[CRAN](https://CRAN.R-project.org) with:

``` r
install.packages("easycensus")
```

Or install the development version from [GitHub](https://github.com/)
with:

``` r
# install.packages("devtools")
devtools::install_github("CoryMcCartan/easycensus")
```

#' Find a decennial or ACS census table with variables of interest
#'
#' This function uses fuzzy matching to help identify tables from the census
#' which contain variables of interest. Matched table codes are printed out,
#' along with the Census-provided table description, the parsed variable names,
#' and example table cells. The website <https://censusreporter.org/> may also
#' be useful in finding variables.
#'
#' @param tables A list of `cens_table` objects, such as is produced by
#'   [cens_parse_tables()].
#' @param ... Variables to look for. These can be length-1 character vectors,
#'   or, for convenience, can be left unquoted (see examples).
#' @param show How many matching tables to show. Increase this to show more
#'   possible matches, at the cost of more output. Negative values will be
#'   converted to positive but will suppress any printing.
#'
#' @returns The codes for the top `show` tables, invisibly if `show` is positive.
#'
#' @examples
#' cens_find_dec("sex", "age")
#' cens_find(tables_sf1, "sex", "age") # same as above
#' cens_find_dec(tenure, race)
#' cens_find_acs("income", "sex", show=3)
#' cens_find_acs("heath care", show=-1)
#'
#' @name cens_find
#' @export
cens_find <- function(tables, ..., show = 4) {
    variables = str_to_upper(vapply(rlang::enquos(...), rlang::as_name, character(1)))
    best = utils::head(match_tables(variables, tables), abs(show))
    if (show > 0) {
        cli_h1("Top {show} matching table{?s}")
        lapply(tables[best], print, all = FALSE)
        invisible(best)
    } else {
        best
    }
}


#' @rdname cens_find
#' @export
cens_find_dec <- function(..., show = 2) {
    variables = str_to_upper(vapply(rlang::enquos(...), rlang::as_name, character(1)))
    best = utils::head(match_tables(variables, tables_sf1), abs(show))
    if (show > 0) {
        cli_h1("Top {show} matching table{?s}")
        lapply(tables_sf1[best], print, all = FALSE)
        invisible(best)
    } else {
        best
    }
}

#' @rdname cens_find
#' @export
cens_find_acs <- function(..., show = 4) {
    variables = str_to_upper(vapply(rlang::enquos(...), rlang::as_name, character(1)))
    best = utils::head(match_tables(variables, tables_acs, complex_pen = 0.1), abs(show))
    if (show > 0) {
        cli_h1("Top {show} matching table{?s}")
        lapply(tables_acs[best], print, all = FALSE)
        invisible(best)
    } else {
        best
    }
}


match_tables = function(variables, table_specs, complex_pen = 0.5) {
    concepts = unlist(lapply(table_specs, function(tbl) {
        paste(tbl$concept, paste(str_to_upper(tbl$dims), collapse = " "))
    }))

    # fuzzy match
    avg_dists_1 = apply(adist(variables, concepts, costs = c(1, 1, 10), partial = TRUE), 2, max)
    avg_dists_2 = colMeans(adist(variables, concepts, costs = c(1, 1, 10), partial = FALSE))
    complexity = str_length(concepts)
    avg_dists = avg_dists_1 /
        mean(avg_dists_1) +
        complex_pen * complexity / mean(complexity) +
        0.1 * avg_dists_2 / mean(avg_dists_2)

    names(sort(avg_dists))
}

# Internal `cens_table` constructor
new_cens_table = function(x) {
    if (!all(c("concept", "tables", "surveys", "dims", "vars") %in% names(x))) {
        cli_abort("Invalid table.")
    }
    structure(x, class = "cens_table")
}

match_survey = function(x) {
    dplyr::case_when(
        x == "acs/acs1" ~ "ACS / 1-year Detailed",
        x == "acs/acs5" ~ "ACS / 5-year Detailed",
        x == "dec/sf1" ~ "Decennial / Summary File 1",
        x == "dec/pl" ~ "Decennial / P.L. 94-171",
        TRUE ~ x
    )
}

#' @export
print.cens_table = function(x, ..., all = FALSE) {
    n_vars = nrow(x$vars)
    cli_text("\n")
    cli_text(
        style_bold(style_inverse(c("\u00a0", x$tables[1], "\u00a0"))), #nbsp
        style_bold(c(" - ", x$concept))
    )

    cli_text("Surveys / Files:")
    lapply(match_survey(x$surveys), cli_alert_success)

    cli_text("Parsed variables:")
    cli_ul(items = str_c("{.field ", x$dims, "}"))

    ex_items = as.matrix(x$vars)[, -1, drop = FALSE]
    if (isTRUE(all)) {
        cli_text("All values:")
    } else {
        cli_text("Example values:")
        ex_items = ex_items[sample.int(n_vars, min(n_vars, 3)), , drop = FALSE]
    }

    if (ncol(ex_items) > 1) {
        ex_items = apply(ex_items, 1, function(x) paste(x, collapse = " / "))
    } else {
        ex_items = ex_items[, 1]
    }

    cli_ul(items = ex_items)
    cli_end()
}

#' Parsed Census SF1 and ACS Tables
#'
#' Contains parsed table information for the 2010 Decennial Summary File 1 and
#' 2019 ACS 5-year and 1-year tables.
#' This parsed information is used internally in [cens_find_dec()],
#' [cens_find_acs()], [cens_get_dec()], and [cens_get_acs()].
#' For other sets of tables, try using [cens_parse_tables()].
#'
#' @format A list of `cens_table` objects, which are just lists with four elements:
#'   - `concept`, a human-readable name
#'   - `tables`, the constituent table codes
#'   - `surveys`, the supported surveys
#'   - `dims`, the parsed names of the dimensions of the tables
#'   - `vars`, a `tibble` with all of the parsed variable values
#'
#' @name tables
NULL

#' @rdname tables
"tables_sf1"

#' @rdname tables
"tables_acs"


#' Attempt to Parse Tables from a Census API
#'
#' Uses the same parsing code as that which generates [tables_sf1] and [tables_acs]
#' See <https://www.census.gov/data/developers/data-sets.html> for a list of
#' APIs and corresponding years, or use [censusapi::listCensusApis()].
#'
#' @param api A Census API programmatic name such as `"acs/acs5"`.
#' @param year The year for the data
#'
#' @returns A list of `cens_table` objects, which are just lists with four elements:
#'   - `concept`, a human-readable name
#'   - `tables`, the constituent table codes
#'   - `surveys`, the supported surveys
#'   - `dims`, the parsed names of the dimensions of the tables
#'   - `vars`, a `tibble` with all of the parsed variable values
#'
#' @examples \dontrun{
#' cens_parse_tables("dec/pl", 2020)
#' }
#'
#' @export
cens_parse_tables = function(api, year) {
    vars = get_survey_vars(api, year)

    dplyr::group_map(vars, parse_table) %>%
        unlist(recursive = FALSE) %>%
        lapply(function(x) {
            x$surveys = api
            x
        }) %>%
        lapply(new_cens_table)
}

# Internal functions to parse Census tables with the API ------

get_survey_vars = function(api, year) {
    raw = censusapi::listCensusMetadata(api, vintage = year, type = "variables") %>%
        as_tibble() %>%
        dplyr::mutate(label = str_trim(.data$label))
    if (str_starts(api, "dec")) {
        raw = dplyr::filter(
            raw,
            str_starts(.data$label, "(!!)?Total:?!!") |
                str_ends(.data$name, "001001"),
            !str_detect(.data$label, "!!Not defined")
        )
    } else if (str_starts(api, "acs")) {
        raw = dplyr::filter(
            raw,
            str_starts(.data$label, "Estimate!!"),
            !str_detect(.data$label, "!!Not defined")
        ) %>%
            dplyr::mutate(label = str_sub(.data$label, 11))
    }
    raw %>%
        dplyr::mutate(table = str_extract(.data$group, "[A-Z]*\\d+")) %>%
        dplyr::group_by(table) %>%
        dplyr::arrange(.data$group)
}

# Handle repeated tables by race
paren_race_re = "\\((WHITE|BLACK|AMERI|ASIAN|NATIVE|SOME OTHER|TWO OR|HISPANIC)[A-Z, ]+\\)"

tidy_names = function(x) {
    str_to_lower(x) %>%
        make.names(unique = TRUE) %>%
        str_replace_all("\\.", "_")
}

# table name to list of nice variables
parse_concept = function(concept) {
    parsed = concept %>%
        str_replace(paren_race_re, "BY RACE-ETHNICITY") %>%
        str_remove_all("AND ") %>%
        str_remove_all("\\(IN 20\\d\\d INFLATION-ADJUSTED DOLLARS\\) ") %>%
        unique() %>%
        str_split(" BY ") %>%
        lapply(tidy_names)
    parsed[[which.max(lengths(parsed))]]
}

# table column/row name to nice label
parse_label_group = function(lbl) {
    if_else(lbl == "", "total", lbl) %>%
        str_remove(":") %>%
        str_remove(" --") %>%
        str_trim() %>%
        str_to_lower()
}


parse_table = function(tbl, key) {
    dim_cats = str_split(tbl$label, "!!", simplify = TRUE)[, drop = FALSE]
    const_cats = apply(dim_cats, 2, dplyr::n_distinct) == 1
    if (nrow(tbl) > 1 && any(const_cats)) {
        dim_cats = dim_cats[, !const_cats, drop = FALSE]
    }
    paren_cats = str_extract(tbl$concept, paren_race_re)
    if (any(!is.na(paren_cats))) {
        new_col = dplyr::coalesce(paren_cats, "") %>%
            str_remove_all("[()]")
        dim_cats = cbind(dim_cats, new_col, deparse.level = 0)
    }

    depth = ncol(dim_cats)
    if (depth == 0L) {
        dim_cats = matrix("")
        depth = 1L
    }

    dim_lbls = parse_concept(unique(tbl$concept))

    short_lbls = str_replace_all(dim_lbls, "_", " ") %>%
        str_remove_all("(or|and|of|the|for|in|,) ") %>%
        str_to_title() %>%
        abbreviate(5, named = FALSE) %>%
        str_to_lower() %>%
        paste(collapse = "_")
    if (depth == length(dim_lbls) + 1L) {
        new_dim = paste0(short_lbls, "_sub")
        dim_lbls = c(dim_lbls, new_dim)
    } else if (depth > length(dim_lbls)) {
        dim_lbls = str_c(short_lbls, "_", seq_len(depth))
    } else if (depth < length(dim_lbls)) {
        dim_lbls = dim_lbls[-seq_len(length(dim_lbls) - depth)]
    }

    vars_tbl = dim_cats %>%
        `colnames<-`(dim_lbls) %>%
        as_tibble() %>%
        dplyr::mutate(variable = tbl$name, .before = 1) %>%
        dplyr::mutate(dplyr::across(-.data$variable, function(x) as.factor(parse_label_group(x)))) %>%
        dplyr::arrange(.data$variable)

    out = list(list(
        concept = tbl$concept[1],
        tables = sort(unique(tbl$group)),
        dims = dim_lbls,
        vars = vars_tbl
    ))
    names(out) = key[[1]][1]
    out
}

#' Download data from a decennial census or ACS table
#'
#' Leverages [censusapi::getCensus()] to download tables of census data. Tables
#' are returned in tidy format, with variables given tidy, human-readable names.
#'
#' @param table The table to download, either as a character vector or a table
#'   object as produced by [cens_find_dec()],  [cens_find_acs()] or
#'   [cens_parse_tables()], or as included in `tables_dec` and `tables_acs`.
#'   Note: some tables are split into A/B/C/etc. versions by race; this function
#'   unifies all of these tables under one code. So, for example, use `P012`,
#'   not `P012A`.
#' @param geo The geographic level to return. One of the machine-readable or
#'   human-readable names listed in the "Details" section of [cens_geo()]. Will
#'   return all matching geographies of this level, as filtered by the further
#'   arguments to `...`.  For example, setting `geo="tract"` is equivalent to
#'   setting `tract="all"`.
#' @param ... Geographies to return, as supported by the Census API. Order
#'   matters here---the first argument will be the geographic level to return
#'   (i.e., it corresponds to the `geo` argument) and additional arguments will
#'   filter the results. Use `"all"`, `"*"`, `NA`, or `TRUE` to return all units
#'   of a particular geography.  See the examples of [cens_geo()] for details.
#' @param year For ACS data, the survey year to get data for.
#' @param survey For ACS data, whether to use the one-year or
#'   five-year survey (the default). Make sure to check availability using
#'   [cens_find_acs()].
#' @param sumfile For decennial data, the summary file to use. SF2 contains more
#'   detailed race and household info.
#' @param api A Census API programmatic name such as `"acs/acs5"`.
#' @param pop_group For decennial data using summary file SF2, the population
#'   group to filter to. See
#'   <https://www2.census.gov/programs-surveys/decennial/2010/technical-documentation/complete-tech-docs/summary-file/sf2.pdf#page=347>.
#' @param check_geo If `TRUE`, validate the provided geographies against the
#'   available geographies from the relevant Census API.
#' @param drop_total Whether to filter out variables which are totals across
#'   another variable. Recommended only after inspection of the underlying
#'   table.
#' @param show_call Whether to show the actual call to the Census API. May be
#'   useful for debugging.
#'
#' @returns A tibble of census data in tidy format, with columns
#'   `GEOID`, `NAME`, `variable` (containing the Census variable code),
#'   `value` or `estimate` in the case of ACS tables,
#'   and additional factor columns specific to the table.
#'
#' @examples \dontrun{
#' cens_get_dec("P3", "state")
#' cens_get_dec(tables_sf1$H2, "state")
#' cens_get_dec("H2", "county", state="WA", drop_total=TRUE)
#'
#' cens_get_acs("B09001", county="King", state="WA")
#' }
#'
#' @name cens_get
NULL


#' @describeIn cens_get Get raw data from another Census Bureau API. Output will
#'   be minimally tidied but will likely require further manipulation.
#' @order 3
#' @export
cens_get_raw <- function(
    table,
    geo = NULL,
    ...,
    year = 2010,
    api = NULL,
    check_geo = FALSE,
    show_call = TRUE
) {
    if (inherits(table, "cens_table")) {
        if (!is.null(api) && !api %in% table$surveys) {
            cli_abort("Table {.field {table$tables[1]}} not found for the provided api {.val {api}}.")
        }
        spec = table
        table = spec$tables[1]
    } else {
        cli_abort(
            "For {.fn cens_get_raw}, {.arg table} must be a
                  {.cls cens_table} object."
        )
    }

    if (is.null(api)) {
        if (length(spec$surveys) > 1) {
            cli_abort(c(
                "Table {.field {table}} exists for multiple Census products.",
                ">" = "Please specify the correct one with {.arg api}"
            ))
        } else {
            api = spec$surveys[1]
        }
    }

    geo_spec = cens_geo(geo, ..., check = check_geo, api = api, year = year)

    caller = rlang::current_call()
    suppressMessages({
        withCallingHandlers(
            {
                d = do.call(
                    dplyr::bind_rows,
                    lapply(spec$tables, function(tbl_code) {
                        load_raw(api, year, tbl_code, geo_spec, show_call = show_call)
                    })
                )
            },
            warning = function(w) {
                if (w$message != "NAs introduced by coercion") {
                    rlang::warn(w$message)
                } else {
                    invokeRestart("muffleWarning")
                }
            },
            error = function(e) {
                rlang::abort(e$message, call = caller)
            }
        )
    })

    tbl_vars = spec$vars
    dplyr::inner_join(d, tbl_vars, by = "variable")
}

#' @describeIn cens_get Get decennial census data.
#' @order 1
#' @export
cens_get_dec <- function(
    table,
    geo = NULL,
    ...,
    sumfile = "sf1",
    pop_group = NULL,
    check_geo = FALSE,
    drop_total = FALSE,
    show_call = FALSE
) {
    if (is.character(table) && length(table) == 1L) {
        if (!table %in% names(tables_sf1)) {
            cli_abort("Table {.field {table}} not found.")
        }
        spec = tables_sf1[[table]]
    } else if (inherits(table, "cens_table")) {
        if (!table$tables[1] %in% names(tables_sf1)) {
            cli_abort("Table {.field {table$tables[1]}} not found for the decennial census.")
        }
        spec = table
        table = spec$tables[1]
    } else {
        cli_abort("{.arg table} must be a character vector or {.cls cens_table}.")
    }

    api = str_c("dec/", sumfile)
    geo_spec = cens_geo(geo, ..., check = check_geo, api = api, year = 2010)

    caller = rlang::current_call()
    suppressMessages({
        withCallingHandlers(
            {
                d = do.call(
                    dplyr::bind_rows,
                    lapply(spec$tables, function(tbl_code) {
                        load_dec(
                            api,
                            2010,
                            tbl_code,
                            geo_spec,
                            pop_group = pop_group,
                            show_call = show_call
                        )
                    })
                )
            },
            warning = function(w) {
                if (w$message != "NAs introduced by coercion") {
                    rlang::warn(w$message)
                } else {
                    invokeRestart("muffleWarning")
                }
            },
            error = function(e) {
                rlang::abort(e$message, call = caller)
            }
        )
    })

    tbl_vars = spec$vars
    if (isTRUE(drop_total)) {
        tbl_vars = dplyr::filter(tbl_vars, dplyr::if_all(-1, function(x) x != "total"))
    }
    dplyr::inner_join(d, tbl_vars, by = "variable")
}

#' @describeIn cens_get Get American Community Survey (ACS) data.
#' @order 2
#' @export
cens_get_acs <- function(
    table,
    geo = NULL,
    ...,
    year = 2019,
    survey = c("acs5", "acs1"),
    check_geo = FALSE,
    drop_total = FALSE,
    show_call = FALSE
) {
    if (is.character(table) && length(table) == 1L) {
        if (!table %in% names(tables_acs)) {
            cli_abort("Table {.field {table}} not found.")
        }
        spec = tables_acs[[table]]
    } else if (inherits(table, "cens_table")) {
        if (!table$tables[1] %in% names(tables_acs)) {
            cli_abort("Table {.field {table$tables[1]}} not found for the ACS.")
        }
        spec = table
        table = spec$tables[1]
    } else {
        cli_abort("{.arg table} must be a character vector or {.cls cens_table}.")
    }

    api = str_c("acs/", match.arg(survey))
    if (!api %in% spec$surveys) {
        cli_abort("{.arg {table}} is not available for {.field {match_survey(api)}}")
    }

    geo_spec = cens_geo(geo, ..., check = check_geo, api = api, year = year)

    caller = rlang::current_call()
    suppressMessages({
        withCallingHandlers(
            {
                d = do.call(
                    dplyr::bind_rows,
                    lapply(spec$tables, function(tbl_code) {
                        load_acs(api, year, tbl_code, geo_spec, show_call = show_call)
                    })
                )
            },
            warning = function(w) {
                if (w$message != "NAs introduced by coercion") {
                    rlang::warn(w$message)
                } else {
                    invokeRestart("muffleWarning")
                }
            },
            error = function(e) {
                rlang::abort(e$message, call = caller)
            }
        )
    })

    tbl_vars = spec$vars
    if (isTRUE(drop_total)) {
        tbl_vars = dplyr::filter(tbl_vars, dplyr::if_all(-1, function(x) x != "total"))
    }
    dplyr::inner_join(d, tbl_vars, by = "variable")
}


# Internal: get Census data and tidy-format
load_raw <- function(api, year, tbl, geo_spec, show_call = TRUE) {
    d = censusapi::getCensus(
        api,
        year,
        key = cens_auth(),
        vars = str_glue("group({tbl})"),
        region = geo_spec$region,
        regionin = geo_spec$regionin,
        show_call = show_call
    ) %>%
        as_tibble()

    d = d[, str_starts(colnames(d), "[a-z]", negate = TRUE)]
    d$GEO_ID = str_sub(d$GEO_ID, 10)

    rgx_tbl = str_replace_all(tbl, "\\d", "\\\\d") # make a regex which matches table variables
    pivot_ids = union(
        # the remaining other columns to use as ID cols in pivoting
        c("GEO_ID", "NAME"),
        colnames(d)[str_starts(colnames(d), rgx_tbl, negate = TRUE)]
    )

    dplyr::select(d, "GEO_ID", "NAME", dplyr::everything(), -dplyr::ends_with("ERR")) %>%
        tidyr::pivot_longer(-dplyr::all_of(pivot_ids), names_to = "variable", values_to = "value") %>%
        dplyr::rename(GEOID = "GEO_ID", race_ethnicity = dplyr::any_of("POPGROUP_TTL"))
}

load_dec <- function(api, year, tbl, geo_spec, pop_group = NULL, show_call = FALSE) {
    d = censusapi::getCensus(
        api,
        year,
        key = cens_auth(),
        vars = str_glue("group({tbl})"),
        region = geo_spec$region,
        regionin = geo_spec$regionin,
        show_call = show_call
    ) %>%
        as_tibble()

    d = d[, str_starts(colnames(d), "[a-z]", negate = TRUE)]
    d$GEO_ID = str_sub(d$GEO_ID, 10)

    pivot_ids = c("GEOID", "NAME")
    if (api == "dec/sf2") {
        if (is.null(pop_group)) {
            pivot_ids = c("GEOID", "NAME", "race_ethnicity")
            d = dplyr::select(d, -.data$POPGROUP) %>%
                dplyr::rename(race_ethnicity = .data$POPGROUP_TTL)
        } else {
            if (!is.character(pop_group) || length(pop_group) != 1) {
                cli_abort("{.arg pop_group} must be a length-1 character vector, or NULL")
            }
            d = dplyr::filter(d, .data$POPGROUP == pop_group) %>%
                dplyr::select(-.data$POPGROUP, -.data$POPGROUP_TTL)
        }
    }

    dplyr::select(d, GEOID = "GEO_ID", "NAME", dplyr::everything(), -dplyr::ends_with("ERR")) %>%
        tidyr::pivot_longer(-dplyr::all_of(pivot_ids), names_to = "variable", values_to = "value")
}

# Internal: get Census data and tidy-format
load_acs <- function(api, year, tbl, geo_spec, show_call = FALSE) {
    d = censusapi::getCensus(
        api,
        year,
        key = cens_auth(),
        vars = str_glue("group({tbl})"),
        region = geo_spec$region,
        regionin = geo_spec$regionin,
        show_call = show_call
    ) %>%
        # dplyr::mutate(dplyr::across(dplyr::where(~ all(is.na(.))), as.numeric)) %>%
        dplyr::mutate(dplyr::across(
            c(
                dplyr::ends_with("_EA"),
                dplyr::ends_with("_MA")
            ),
            as.character
        )) %>%
        as_tibble()

    d = d[, str_starts(colnames(d), "[a-z]", negate = TRUE)]
    d$GEO_ID = str_sub(d$GEO_ID, 10)

    dplyr::select(d, GEOID = "GEO_ID", "NAME", dplyr::everything(), -dplyr::ends_with("ERR")) %>%
        tidyr::pivot_longer(
            -c("GEOID", "NAME"),
            names_pattern = "([A-Z][0-9]+[A-Z]?[0-9_]+)([A-Z]+)",
            names_to = c("variable", ".value")
        ) %>%
        dplyr::mutate(
            variable = str_c(.data$variable, "E"),
            E = dplyr::case_when(
                .data$EA == "-" ~ NA_real_,
                .data$EA == "N" ~ NA_real_,
                .data$EA == "(X)" ~ NA_real_,
                is.na(.data$EA) ~ .data$E,
                TRUE ~ .data$E
            ),
            M = dplyr::case_when(
                .data$MA == "**" ~ NA_real_,
                .data$MA == "***" ~ NA_real_,
                .data$MA == "*****" ~ 0,
                .data$MA == "N" ~ NA_real_,
                .data$MA == "(X)" ~ NA_real_,
                is.na(.data$MA) ~ .data$M,
                TRUE ~ .data$M
            ),
            value = estimate(.data$E, .data$M)
        ) %>%
        dplyr::select("GEOID", "NAME", "variable", estimate = "value")
}

test_that("decennial tables can be downloaded", {
    skip_if(is.null(cens_auth_env()), "Census API not available")

    d = cens_get_dec("P4", "county", state = "RI")
    expect_s3_class(d, "data.frame")
    expect_equal(length(unique(d$GEOID)), 5)
    expect_true(any(grepl("hispanic", names(d))))
    expect_equal(sum(d$value), 1052567)
})

test_that("ACS tables can be downloaded", {
    skip_if(is.null(cens_auth_env()), "Census API not available")

    d = cens_get_acs("B09001", "county", state = "RI")
    expect_s3_class(d, "data.frame")
    expect_equal(length(unique(d$GEOID)), 5)
    expect_true(any(grepl("age", names(d))))
    expect_s3_class(d$estimate, "estimate")
    x = sum(d$estimate)
    expect_equal(get_est(x), 621325)
    expect_equal(get_se(x), 2515, tolerance = 0.01)

    d = cens_get_acs("B25077", "state", year = 2021)
    expect_s3_class(d, "data.frame")
    expect_s3_class(d$estimate, "estimate")

    d = cens_get_acs("B25075", "state", year = 2021)
    expect_s3_class(d, "data.frame")
    expect_s3_class(d$estimate, "estimate")
})

#' @keywords internal
"_PACKAGE"

# The following block is used by usethis to automatically manage
# roxygen namespace tags. Modify with care!
#' @import stringr
#' @import cli
#' @importFrom dplyr as_tibble if_else %>%
#' @importFrom rlang .data .env
#' @importFrom utils data adist
#' @importFrom stats qnorm rnorm
## usethis namespace: start
## usethis namespace: end
NULL

# load table names for autocomplete
.onLoad <- function(libname, pkgname) {
    data("tables_sf1", package = pkgname, envir = parent.env(environment()))
    data("tables_acs", package = pkgname, envir = parent.env(environment()))
}

#' Authorize use of the Census API
#'
#' Tries environment variables `CENSUS_API_KEY` and `CENSUS_KEY`, in that order.
#' If none is found and R is used in interactive mode, will prompt the user for
#' a key.
#'
#' @returns a Census API key
cens_auth = function() {
    if (!is.null(key <- cens_auth_env())) {
        key
    } else if (interactive()) {
        cli_alert_info("A Census API key is required to download Census data.")
        url <- "http://api.census.gov/data/key_signup.html"
        cli_text("Sign up at ", cli::style_hyperlink(text = str_c("<", url, ">"), url = url))
        key = readline("Enter your key here: ")
        if (nchar(key) != 40) {
            cli_abort("A valid Census API key must be provided.", .envir = parent.frame())
        } else {
            cli_inform(c(
                ">" = "Use the key automatically in the future by adding
                         the following line to your {.path .Renviron}:"
            ))
            cli_text("CENSUS_API_KEY=\"{key}\"")
        }
        key
    } else {
        cli_abort("A valid Census API key must be provided.", .envir = parent.frame())
    }
}

# Internal getter. Returns NULL if no key
cens_auth_env = function() {
    if ((key <- Sys.getenv("CENSUS_API_KEY")) != "") {
        key
    } else if ((key <- Sys.getenv("CENSUS_KEY")) != "") {
        key
    } else {
        NULL
    }
}

#' Helper function to sum over nuisance variables
#'
#' For ACS data, margins of error will be updated appropriately, using
#' the functionality in [estimate()].
#'
#' @param data The output of [cens_get_dec()] or [cens_get_acs()]
#' @param ... The variables of interest, which will be kept. Remaining variables
#'   will be marginalized out.
#'
#' @return A new data frame that has had [dplyr::group_by()] and
#'   [dplyr::summarize()] applied.
#'
#' @examples \dontrun{
#' d_cens = cens_get_acs("state", "B25042")
#' cens_margin_to(d_cens, bedrooms)
#' }
#' @export
cens_margin_to = function(data, ...) {
    dplyr::summarize(
        dplyr::group_by(data, .data$GEOID, .data$NAME, ...),
        dplyr::across(dplyr::any_of(c("value", "estimate")), sum),
        .groups = "drop"
    )
}

#' Tidy labels in census tables
#'
#' Some table labels are quite verbose, and users will often want to shorten them.
#' These functions make tidying common types of labels easy.
#' Most produce straightforward output, but there are several more generic tidiers:
#' * [tidy_simplify()] attempts to simplify labels by removing words common to all labels.
#' * [tidy_parens()] attempts to simplify labels by removing all terms in parentheses.
#' * [tidy_race_detailed()] creates logical columns for each of the six racial categories.
#'
#' @param x A factor, which will be re-leveled. Character vectors will be converted to factors.
#' @param x2,x3 Additional character columns containing detailed information for certain variables (e.g. detailed race)
#'
#' @return A re-leveled factor, except for [tidy_age_bins()], which by default
#'   returns a data frame with columns `age_from` and `age_to` (inclusive).
#'
#' @examples
#' ex_race_long = c("american indian and alaska native alone", "asian alone",
#'     "black or african american alone", "hispanic or latino",
#'     "native hawaiian and other pacific islander alone",
#'     "some other race alone", "total", "two or more races",
#'     "white alone", "white alone, not hispanic or latino")
#' tidy_race(ex_race_long)
#'
#' tidy_age_bins(c("10 to 14 years", "21 years", "85 years and over"))
#'
#' tidy_parens(c("label one (fake)", "label two (fake)"))
#' tidy_simplify(c("label one (fake)", "label two (fake)"))
#'
#' \dontrun{ # requires API key
#' d = cens_get_acs("B02003", "us", year=2019, survey="acs1")
#' dplyr::mutate(d, tidy_race_detailed(dtldr_1, dtldr_2, dtldr_3))
#' }
#'
#' @name tidiers
NULL

check_fc = function(x) {
    if (is.character(x)) {
        x = as.factor(x)
    }
    if (!is.factor(x)) {
        cli_abort("{.arg x} must be a {.cls factor} or {.cls character}", call = rlang::caller_env())
    }
    x
}

#' @rdname tidiers
#' @export
tidy_race = function(x) {
    x = check_fc(x)
    xlev = str_squish(str_remove(levels(x), "household(er)?"))
    levels(x) = labs_race_ethnicity[xlev]
    x
}

#' @rdname tidiers
#' @export
tidy_race_detailed = function(x, x2, x3) {
    x = check_fc(x)
    x2 = check_fc(x2)
    x3 = as.character(check_fc(x3))
    xlev = str_squish(str_remove(levels(x), "household(er)?"))
    xlev2 = str_squish(str_remove(levels(x2), "household(er)?"))
    x = xlev[as.integer(x)]
    x2 = xlev2[as.integer(x2)]

    drop_row = (x == "total" | x2 == "total" | (str_starts(x2, "population of") & x3 == "total"))
    x[drop_row] = NA_character_
    x2[drop_row] = NA_character_
    x3[drop_row] = NA_character_
    idx_alone = which(x3 == "total")
    x3[idx_alone] = str_remove(x2[idx_alone], " alone")

    races = c(
        white = "white",
        black = "black or african american",
        aian = "american indian and alaska native",
        asian = "asian",
        nhpi = "native hawaiian and other pacific islander",
        other = "some other race"
    )

    x3s = str_split(x3, "; ", simplify = FALSE)

    out = list()
    for (i in seq_along(races)) {
        out[[names(races)[i]]] = vapply(x3s, function(x) any(x == races[i]), logical(1))
    }

    as_tibble(out)
}

#' @rdname tidiers
#' @export
tidy_ethnicity = function(x) {
    x = check_fc(x)
    xlev = str_squish(str_remove(levels(x), "household(er)?"))
    levels(x) = labs_ethnicity[xlev]
    x
}

#' @rdname tidiers
#' @export
tidy_age = function(x) {
    x = check_fc(x)
    xlev = levels(x)
    xlev = str_replace(str_remove(xlev, "years?"), "under 1", "0")
    levels(x) = str_squish(xlev)
    x
}

#' @param as_factor if `TRUE`, return a factor with levels of the form `[10,14]`.
#' @rdname tidiers
#' @export
tidy_age_bins = function(x, as_factor = FALSE) {
    x = check_fc(x)
    xlev = str_remove(levels(x), "householder")
    xlev = str_squish(str_remove(xlev, "years?"))
    xlev = str_replace(xlev, "under", "0 to")
    xlev = str_replace(xlev, "total", "0 to Inf")
    xlev = str_replace(xlev, "over", "Inf")

    m = str_split(xlev, " (and|to) ", simplify = TRUE)
    if (ncol(m) != 2) {
        cli_abort("Problem parsing age categories: {.code {levels(x)}}")
    }
    m[m[, 2] == "", 2] = m[m[, 2] == "", 1]
    m = matrix(as.numeric(m), nrow = nrow(m))
    colnames(m) = c("age_from", "age_to")

    if (isFALSE(as_factor)) {
        as_tibble(m[as.integer(x), , drop = FALSE])
    } else {
        close_br = dplyr::if_else(m[, 2] == Inf, ")", "]")
        levels(x) = str_glue("[{m[,1]},{m[,2]}{close_br}")
        x
    }
}


#' @param as_factor if `TRUE`, return a factor with levels of the form `[35,40]`.
#' @rdname tidiers
#' @export
tidy_income_bins = function(x, as_factor = FALSE) {
    x = check_fc(x)
    xlev = str_squish(str_remove_all(levels(x), "[$,]"))
    xlev = str_replace(xlev, "less than", "0 to")
    xlev = str_replace(xlev, "total", "0 to Inf")
    xlev = str_replace(xlev, "or more", "to Inf")

    m = str_split(xlev, " to ", simplify = TRUE)
    if (ncol(m) != 2) {
        cli_abort("Problem parsing income categories: {.code {levels(x)}}")
    }
    m[m[, 2] == "", 2] = m[m[, 2] == "", 1]
    nine_rows = str_ends(m[, 2], "999")
    m = matrix(as.numeric(m), nrow = nrow(m))
    m[nine_rows, 2] = m[nine_rows, 2] + 1
    m = m / 1e3
    colnames(m) = c("inc_from", "inc_to")

    if (isFALSE(as_factor)) {
        as_tibble(m[as.integer(x), , drop = FALSE])
    } else {
        levels(x) = str_glue("$[{m[,1]},{m[,2]})k")
        x
    }
}

#' @rdname tidiers
#' @export
tidy_simplify = function(x) {
    x = check_fc(x)
    xlev = str_squish(levels(x))
    common_words = Reduce(intersect, str_split(xlev, " "))
    common_regex = str_c("(", paste("\\Q", common_words, "\\E", sep = "", collapse = "|"), ")")
    xlev = str_squish(str_remove_all(xlev, common_regex))
    levels(x) = xlev
    x
}

#' @rdname tidiers
#' @export
tidy_parens = function(x) {
    x = check_fc(x)
    levels(x) = str_squish(str_remove_all(levels(x), "\\(.+\\)"))
    x
}


labs_race_ethnicity = c(
    "total" = "total",
    "population of one race" = "one",
    "white alone" = "white",
    "black or african american alone" = "black",
    "american indian and alaska native alone" = "aian",
    "asian alone" = "asian",
    "native hawaiian and other pacific islander alone" = "nhpi",
    "some other race alone" = "other",
    "two or more races" = "two",
    "population of two or more races" = "two",
    "population of two races" = "two",
    "population of three races" = "two",
    "population of four races" = "two",
    "population of five races" = "two",
    "population of six races" = "two",
    "hispanic or latino" = "hisp",
    "white alone, not hispanic or latino" = "white_nh"
)

labs_ethnicity = c("hispanic or latino" = "hisp", "not hispanic or latino" = "nonhisp")

test_that("tidiers produce no warnings", {
    expect_warning(
        {
            tidy_age(tables_sf1$P12$vars$age)
            tidy_age_bins(tables_sf1$P12$vars$age)
            tidy_age_bins(tables_sf1$P12$vars$age, as_factor = TRUE)
            tidy_race(tables_sf1$P12$vars$race_ethnicity)
            tidy_race(tables_sf1$P8$vars$race_1)
        },
        regex = NA
    ) # aka expect _no_ warning
})

test_that("generic tidiers work as expected", {
    test_labs = c("label one (fake)", "label two (fake)")
    expect_equal(levels(tidy_simplify(test_labs)), c("one", "two"))
    expect_equal(levels(tidy_parens(test_labs)), c("label one", "label two"))
})

#' Construct a Geography Specification for Census Data
#'
#' Currently used mostly internally.
#' Builds a Census API-formatted specification of which geographies to download
#' data for. State and county names (or postal abbreviations) are partially
#' matched to existing tables, for ease of use. Other geographies should be
#' specified with Census GEOIDs. The `usgazeteer` package, available with
#' `remotes::install_github("bhaskarvk/usgazetteer")`, may be useful in finding
#' GEOIDs for other geographies. Consult the "geography" sections of each API
#' at <https://www.census.gov/data/developers/data-sets.html> for information on
#' which geographic specifiers may be provided in combination with others.
#'
#' Supported geography arguments:
#'
#' ```{r geos, echo=FALSE, results="asis"}
#' code_names = names(geo)
#' cens_names = unlist(geo)
#' paren_labels = dplyr::if_else(
#'     cens_names == code_names, "",
#'     stringr::str_c(" (", str_to_title(cens_names), ")")
#'     )
#' cat(stringr::str_glue("* `{code_names}`{paren_labels}"), sep="\n")
#' ```
#'
#' @param geo The geographic level to return. One of the machine-readable or
#'   human-readable names listed in the "Details" section. Will return all
#'   matching geographies of this level, as filtered by the further arguments to
#'   `...`.  For example, setting `geo="tract"` is equivalent to setting
#'   `tract="all"`.
#' @param ... Geographies to return, as supported by the Census API. Order
#'   matters here---the first argument will be the geographic level to return
#'   (i.e., it corresponds to the `geo` argument) and additional arguments will
#'   filter the results. Use `"all"`, `"*"`, `NA`, or `TRUE` to return all units
#'   of a particular geography.  See the examples for details.
#' @param check If `TRUE`, validate the provided geographies against the
#'   available geographies from the relevant Census API. Requires the `api` and
#'   `year` arguments to be specified.
#' @param api A Census API programmatic name such as `"acs/acs5"`.
#' @param year The year for the data
#'
#' @returns A list with two elements, `region` and `regionin`, which together
#'   specify a valid Census API geography argument.
#'
#' @examples \dontrun{
#' cens_geo(state="WA")
#' cens_geo("county", state="WA") # equivalent to `cens_geo(county="all", state="WA")`
#' cens_geo(county="King", state="Wash")
#' cens_geo(zcta="02138", check=FALSE)
#' cens_geo(zcta=NA, state="WA", check=FALSE)
#' cens_geo("zcta", state="WA", check=FALSE)
#' cens_geo(cd="09", state="WA", check=FALSE)
#' cens_geo("county_part", state="WA", cd="09", check=FALSE)
#' }
#'
#' @export
cens_geo <- function(geo = NULL, ..., check = TRUE, api = "acs/acs5", year = 2019) {
    geo_in = list(...)
    geo_in = lapply(geo_in, regularize_wild)
    names(geo_in) = vapply(names(geo_in), match_geo, character(1))

    # parse state and county
    has_state = FALSE
    if ("state" %in% names(geo_in)) {
        has_state = TRUE
        geo_in$state = fips_state(geo_in$state)
    }
    idx_cty = which(names(geo_in) == "county" | names(geo_in) == "county (or part)")
    if (length(idx_cty) > 0 && !has_state) {
        cli_abort("{.arg state} must be provided if {.arg county} is specified.")
    }
    for (i in idx_cty) {
        geo_in[[i]] = fips_county(geo_in[[i]], geo_in$state)
    }

    # find the 'for' variable
    if (is.null(geo)) {
        if (length(geo_in) == 0) {
            cli_abort("Must provide at least one geographic level.")
        }
        geo_for = match_geo(names(geo_in)[[1]])
        geo_for_val = geo_in[[1]]
        geo_in = geo_in[-1] # remove first from list
    } else {
        if (!is.character(geo) && length(geo) == 1) {
            cli_abort("{.arg geo} should be a string.")
        }
        geo_for = match_geo(geo)
        geo_for_val = "*"
    }

    if (isTRUE(check)) {
        # match geographies to available
        d_geo = as_tibble(censusapi::listCensusMetadata(name = api, vintage = year, type = "geographies", key = cens_auth()))
        idx_for = which(geo_for == d_geo$name)
        if (length(idx_for) == 0) {
            cli_abort("Geography level {.val {geo_for}} not found.")
        }
        matched = 0L

        req_test = paste0(sort(names(geo_in)), collapse = "|")
        # look for perfect 'requires' match
        req = vapply(d_geo$requires, function(x) paste0(sort(x), collapse = "|"), character(1))
        idx_in = which(req_test == req)
        idx_both = intersect(idx_for, idx_in)
        matched = length(idx_both)

        # look for match without wildcards
        if (matched == 0L) {
            req_soft = character(nrow(d_geo))
            for (i in seq_along(req_soft)) {
                req_soft[i] = paste0(setdiff(d_geo$requires[[i]], d_geo$wildcard[[i]]), collapse = "|")
            }
            idx_in = which(req_test == req_soft)
            idx_both = intersect(idx_for, idx_in)
            matched = length(idx_both)
            if (matched == 1L) {
                match_geo = d_geo[idx_both, ]
                for (x in d_geo$wildcard[[idx_both]]) {
                    geo_in[[x]] = "*"
                }
            }
        }

        hyper = style_hyperlink(
            "<https://www.census.gov/data/developers/data-sets.html>",
            "https://www.census.gov/data/developers/data-sets.html"
        )
        if (matched == 0L) {
            cli_abort(c(
                "Geography combination {.val {c(geo_for, names(geo_in))}} not found.",
                ">" = "Check {hyper} and make sure you are using a valid combination
                        for your survey and file."
            ))
        }
        if (matched > 1L) {
            cli_abort(c(
                "Multiple geography combinations found for {.val {c(geo_for, names(geo_in))}}.",
                ">" = "Check {hyper} and make sure you are using a valid combination
                        for your survey and file."
            ))
        }
    }

    if (length(geo_in) > 0) {
        list(
            region = str_c(geo_for, ":", geo_for_val),
            regionin = paste0(str_c(names(geo_in), ":", geo_in), collapse = "+")
        )
    } else {
        list(region = str_c(geo_for, ":", geo_for_val), regionin = NULL)
    }
}

# Internal: match a geography level
match_geo <- function(g) {
    geo_levels = unlist(geo)
    match = dplyr::coalesce(pmatch(g, names(geo)), pmatch(g, geo_levels))
    if (is.na(match)) {
        cli_abort("Geography level {.val {g}} not found.", call = parent.frame())
    } else {
        geo_levels[match]
    }
}

regularize_wild <- function(x) {
    if_else(x == "all" | x == "*" | isTRUE(x) | is.na(x), "*", as.character(x))
}

# Internal: match state to FIPS
fips_state <- function(state) {
    if (is.numeric(state) && state > 0 && state < 1e2) {
        # already a fips code
        state = str_pad(as.character(as.integer(state)), 2, pad = "0")
    } else {
        state = str_to_lower(state)
    }

    match = dplyr::coalesce(
        pmatch(state, states$fips),
        pmatch(state, str_to_lower(states$abbr)),
        pmatch(state, str_to_lower(states$name))
    )

    if (is.na(match)) {
        cli_abort("No match found for {.val {state}}.", .envir = parent.frame())
    } else {
        states$fips[match]
    }
}

# Internal: match county to FIPS
fips_county <- function(county, state_fips) {
    if (is.numeric(county) && county > 0 && county < 1e5) {
        # already a fips code
        county = str_pad(as.character(as.integer(county)), 3, pad = "0")
    } else {
        county = str_to_lower(county)
    }

    idx_st = which(counties$state == state_fips)
    names = str_to_lower(counties$name[idx_st])
    fips = counties$county[idx_st]

    match = dplyr::coalesce(pmatch(county, fips), pmatch(county, names))

    if (is.na(match)) {
        # cli_abort("No match found for {.val {county}}.", .envir=parent.frame())
        cli_abort("No match found for {.val {county}}.")
    } else {
        fips[match]
    }
}

#' Internal vctrs methods
#'
#' @import vctrs
#' @keywords internal
#' @name estimate-vctrs
NULL

# Constructors and type check ---------

#' Estimate class
#'
#' A numeric vector that stores margin-of-error information along with it.
#' The margin of error will update through basic arithmetic operations, using a
#' first-order Taylor series approximation. The implicit assumption is that the
#' errors in each value are uncorrelated. If in fact there is correlation, the
#' margins of error could be wildly under- or over-estimated.
#'
#' @param x A numeric vector containing the estimate(s).
#' @param se A numeric vector containing the standard error(s) for the
#'   estimate(s). Users should supply either `se` or `moe` and `conf`.
#' @param moe A numeric vector containing the margin(s) of error. Users should
#'   supply either `se` or `moe` and `conf`.
#' @param conf The confidence level to use in converting the margin of error to
#'   a standard error. Defaults to 90%, which is what the Census Bureau uses for
#'   ACS estimates.
#'
#' @returns An `estimate` vector.
#'
#' @examples
#' estimate(5, 2) # 5 with std. error  2
#' estimate(15, moe=3) - estimate(5, moe=4)
#' estimate(1:4, 0.1) * estimate(1, 0.1)
#'
#' @export
estimate = function(x, se = NULL, moe = NULL, conf = 0.9) {
    # length-0 case
    if (missing(x) || length(x) == 0) {
        return(new_estimate())
    }

    if (is.null(se) + is.null(moe) != 1) {
        cli_abort("Exactly one of  {.arg se} or {.arg moe} must be provided.")
    }

    if (!is.null(moe)) {
        if (any(moe < 0, na.rm = TRUE)) {
            cli_abort("{.arg moe} is a margin of error and must be nonnegative.")
        }
        if (conf <= 0 || conf >= 1) {
            cli_abort("{.arg conf} must be between 0 and 1.")
        }

        se = moe / conf_z(conf)
    }

    if (!is.numeric(x) || !is.numeric(se)) {
        cli_abort("All arguments must be numeric.")
    }

    if (any(se < 0, na.rm = TRUE)) {
        cli_abort("{.arg se} is a standard error and must be nonnegative.")
    }

    do.call(new_estimate, vctrs::vec_recycle_common(est = x, se = se))
}

# internal constructor
new_estimate = function(est = double(), se = double()) {
    vctrs::new_rcrd(list(est = est, se = se), class = "estimate")
}

#' @export
#' @rdname estimate
is_estimate = function(x) {
    inherits(x, "estimate")
}

#' Extract estimates, standard errors, and margins of error
#'
#' Getter functions for [estimate()] vectors.
#'
#' @param x An [estimate] vector.
#' @param conf The confidence level to use in constructing the margin of error.
#'
#' @return An [estimate] vector.
#'
#' @examples
#' x = estimate(1, 0.1)
#' get_est(x)
#' get_moe(x)
#'
#' @rdname est_extract
#' @export
get_est = function(x) {
    if (!is_estimate(x)) {
        cli_abort("{.arg x} must be an {.cls estimate}")
    }
    field(x, "est")
}

#' @rdname est_extract
#' @export
get_se = function(x) {
    if (!is_estimate(x)) {
        cli_abort("{.arg x} must be an {.cls estimate}")
    }
    field(x, "se")
}

#' @rdname est_extract
#' @export
get_moe = function(x, conf = 0.9) {
    if (!is_estimate(x)) {
        cli_abort("{.arg x} must be an {.cls estimate}")
    }
    field(x, "se") * conf_z(conf)
}

#' Convert an [estimate] to an `rvar`
#'
#' The [posterior::rvar] class may be useful in handling standard errors for
#' more complicated mathematical expressions. This function assumes a Normal
#' distribution centered on the estimate, with standard deviation equal to the
#' standard error of the estimate. The `posterior` package is required for this
#' function.
#'
#' @param x An [estimate] vector.
#' @param n How many samples to draw.
#'
#' @return A [posterior::rvar] vector.
#'
#' @examples
#' x = estimate(1, 0.1)
#' if (requireNamespace("posterior", quietly=TRUE)) {
#'     rv_x = to_rvar(x)
#'     (rv_x^2 / rv_x) - rv_x # std. errors zero (correct)
#'     x^2 / x - x # std. errors not zero
#' }
#'
#' @rdname est_extract
#' @export
to_rvar = function(x, n = 500) {
    rlang::check_installed("posterior")
    posterior::rvar_rng(rnorm, n = length(x), mean = field(x, "est"), sd = field(x, "se"))
}

# helper throughout
conf_z = function(p) -qnorm((1 - p) / 2)


# Coercion / casting --------

#' @export
vec_ptype2.estimate.estimate = function(x, y, ...) new_estimate()

#' @export
vec_ptype2.double.estimate = function(x, y, ...) new_estimate()
#' @export
vec_ptype2.estimate.double = function(x, y, ...) new_estimate()

#' @export
vec_ptype2.integer.estimate = function(x, y, ...) new_estimate()
#' @export
vec_ptype2.estimate.integer = function(x, y, ...) new_estimate()


#' @export
vec_cast.estimate.estimate = function(x, to, ...) x

#' @export
vec_cast.double.estimate = function(x, to, ...) field(x, "est")
#' @export
vec_cast.estimate.double = function(x, to, ...) estimate(x, se = 0.0)

#' @export
vec_cast.estimate.integer = function(x, to, ...) estimate(as.double(x), se = 0.0)

#' @rdname estimate
#' @export
as_estimate <- function(x) {
    vec_cast(x, new_estimate())
}


# Equality / comparison -------

#' @export
vec_proxy_compare.estimate = function(x, ...) field(x, "est")


# Math -------

#' @method vec_arith estimate
#' @export
vec_arith.estimate = function(op, x, y, ...) {
    UseMethod("vec_arith.estimate", y)
}
#' @method vec_arith.estimate default
#' @export
vec_arith.estimate.default = function(op, x, y, ...) {
    stop_incompatible_op(op, x, y)
}

#' @method vec_arith.estimate estimate
#' @export
vec_arith.estimate.estimate = function(op, x, y, ...) {
    est_x = field(x, "est")
    est_y = field(y, "est")
    se_x = field(x, "se")
    se_y = field(y, "se")

    switch(
        op,
        "+" = new_estimate(est_x + est_y, sqrt(se_x^2 + se_y^2)),
        "-" = new_estimate(est_x - est_y, sqrt(se_x^2 + se_y^2)),
        "*" = new_estimate(est_x * est_y, sqrt(est_x^2 * se_x^2 + est_y^2 * se_y^2)),
        "/" = new_estimate(est_x / est_y, sqrt(se_x^2 + (est_x / est_y)^2 * se_y^2)),
        stop_incompatible_op(op, x, y)
    )
}

#' @method vec_arith.estimate numeric
#' @export
vec_arith.estimate.numeric = function(op, x, y, ...) {
    recyc = vec_recycle_common(x = x, y = y)
    est_x = field(recyc$x, "est")
    se_x = field(recyc$x, "se")
    y = recyc$y
    n = length(y)

    switch(
        op,
        "+" = new_estimate(est_x + y, se_x),
        "-" = new_estimate(est_x - y, se_x),
        "*" = new_estimate(est_x * y, se_x * y),
        "/" = new_estimate(est_x / y, se_x / y),
        "^" = if_else(
            y == 0,
            new_estimate(rep(1, n), rep(0, n)),
            if_else(y == 1, recyc$x, new_estimate(est_x^y, y * est_x^(y - 1) * se_x))
        ),
        stop_incompatible_op(op, x, y)
    )
}
#' @method vec_arith.numeric estimate
#' @export
vec_arith.numeric.estimate = function(op, x, y, ...) {
    recyc = vec_recycle_common(x = x, y = y)
    est_y = field(recyc$y, "est")
    se_y = field(recyc$y, "se")
    x = recyc$x
    n = length(x)

    switch(
        op,
        "+" = new_estimate(est_y + x, se_y),
        "-" = new_estimate(est_y - x, se_y),
        "*" = new_estimate(est_y * x, se_y * x),
        "/" = new_estimate(est_y / x, se_y / x),
        "^" = if_else(
            x == 0,
            new_estimate(rep(0, n), rep(0, n)),
            new_estimate(est_y^x, est_y^x * log(x) * se_y)
        ),
        stop_incompatible_op(op, x, y)
    )
}

#' @method vec_arith.estimate MISSING
#' @export
vec_arith.estimate.MISSING <- function(op, x, y, ...) {
    switch(
        op,
        `-` = new_estimate(-field(x, "est"), field(x, "se")),
        `+` = x,
        stop_incompatible_op(op, x, y)
    )
}

#' @export
vec_math.estimate <- function(.fn, .x, ...) {
    est = field(.x, "est")
    se = field(.x, "se")
    n = length(est)

    switch(
        .fn,
        sum = new_estimate(sum(est), sqrt(sum(se^2))),
        mean = new_estimate(mean(est), sqrt(sum(se^2)) / n),
        abs = {
            abs_est = abs(est)
            if (all(abs_est >= 3 * se, na.rm = T)) {
                new_estimate(abs_est, se)
            } else {
                cli_warn("Some values too close to zero to calculate absolute value errors.")
                abs_est
            }
        },
        sign = {
            if (all(abs(est) >= 3 * se, na.rm = T)) {
                new_estimate(sign(est), rep(0, n))
            } else {
                cli_warn("Some values too close to zero to calculate absolute value errors.")
                abs_est
            }
        },
        sqrt = new_estimate(sqrt(est), 0.5 * se / sqrt(est)),
        exp = new_estimate(exp(est), exp(est) * se),
        expm1 = new_estimate(expm1(est), exp(est) * se),
        log = new_estimate(log(est), se / est),
        log1p = new_estimate(log1p(est), se / (1 + est)),
        cos = new_estimate(cos(est), -sin(est) * se),
        sin = new_estimate(sin(est), cos(est) * se),
        tan = new_estimate(cos(est), se / cos(est)^2),
        {
            cli_warn(c(
                "Function {.fn {.fn}} not supported.",
                ">" = "Coerce to numeric to suppress this warning."
            ))
            vec_math_base(.fn, est)
        }
    )
}


#' Specialized margin-of-error calculations
#'
#' Proportions and percent-change-over-time calculations require different
#' standard error calculations.
#'
#' @param x,y An [estimate] vector. For `est_pct_chg()`, calculates the % change
#'   from `x` to `y` (i.e., \eqn{(y-x)/x})
#'
#' @return An [estimate] vector.
#'
#' @examples
#' x = estimate(1, 0.1)
#' y = estimate(1.5, 0.1)
#' est_prop(x, y)
#' est_pct_chg(x, y)
#'
#' @rdname est_special
#' @export
est_prop = function(x, y) {
    est_x = field(x, "est")
    est_y = field(y, "est")
    se_x = field(x, "se")
    se_y = field(y, "se")

    new_var = se_x^2 - (est_x / est_y)^2 * se_y^2
    new_var = if_else(new_var >= 0, new_var, se_x^2 + (est_x / est_y)^2 * se_y^2)
    new_estimate(est_x / est_y, sqrt(new_var))
}

#' @rdname est_special
#' @export
est_pct_chg = function(x, y) {
    y / x - 1
}


# Formatting ---------

#' Format an estimate
#'
#' Format an estimate for pretty printing
#'
#' @param x An [estimate] vector
#' @param conf The confidence level to use in converting the margin of error to
#'   a standard error. Defaults to 90%, which is what the Census Bureau uses for
#'   ACS estimates.
#' @param digits The number of dig
#' @param trim logical; if `FALSE`, logical, numeric and complex values are
#'   right-justified to a common width: if `TRUE` the leading blanks for
#'   justification are suppressed.
#' @param ... Ignored.
#' @param formatter the formatting function to use internally
#'
#' @export
format.estimate = function(x, conf = 0.9, digits = 2, trim = FALSE, ..., formatter = fmt_plain) {
    out = formatter(x, conf = conf, digits = digits, trim = trim)
    out[is.na(x)] = NA_character_
    out
}

fmt_plain = function(x, conf = 0.9, digits = 2, trim = FALSE) {
    est = fmt_est(field(x, "est"), digits = digits, trim = trim)
    moe = fmt_moe(field(x, "se"), conf = conf, digits = digits, trim = trim)
    str_c(est, " \u00B1 ", moe)
}

fmt_color = function(x, conf = 0.9, digits = 2, trim = FALSE) {
    est = fmt_est(field(x, "est"), digits = digits, trim = trim)
    moe = fmt_moe(field(x, "se"), conf = conf, digits = digits, trim = trim)
    str_c(est, pillar::style_subtle(str_c(" \u00B1 ", moe)))
}

fmt_est = function(x, digits = 2, trim = FALSE) {
    format(x, justify = "right", digits = digits, scientific = 2, trim = trim)
}

fmt_moe = function(x, conf = 0.9, digits = 2, trim = FALSE) {
    format(conf_z(conf) * x, justify = "left", digits = digits, scientific = 2, trim = trim)
}


#' @export
vec_ptype_abbr.estimate = function(x, ..., prefix_named = FALSE, suffix_shape = TRUE) "est"

#' @importFrom pillar pillar_shaft
#' @export
pillar_shaft.estimate <- function(x, ...) {
    out <- format(x, formatter = fmt_color)
    pillar::new_pillar_shaft_simple(out, align = "right", min_width = 9)
}

test_that("constructor works", {
    x1 = estimate(1:3, 2)
    expect_s3_class(x1, "estimate")

    expect_s3_class(estimate(), "estimate")

    expect_error(estimate(1:3, -2), "nonnegative")
    expect_error(estimate(1:3, se = 1, moe = 1.96), "Exactly one")
    expect_error(estimate(1:3, se = double()), "recycle")

    x2 = estimate(1, moe = 1, conf = 0.95)
    expect_equal(vctrs::field(x2, "se"), 1 / qnorm(0.975))
})

test_that("printing works", {
    x1 = estimate(pi, exp(-1))
    expect_equal(format(x1), "3.1 ± 0.61")
    expect_equal(format(x1, digits = 6), "3.14159 ± 0.605108")
    expect_equal(format(x1, conf = 0.5), "3.1 ± 0.25")
})

test_that("coercion and comparison work", {
    x1 = estimate(1, 2)
    x2 = estimate(-1, 5)
    expect_true(x2 < x1)
    expect_s3_class(c(x1, x2), "estimate")
    expect_length(c(x1, x2), 2)
    expect_s3_class(vctrs::vec_c(x1, 5.0), "estimate")
    expect_s3_class(vctrs::vec_c(5.0, x1), "estimate")
    expect_s3_class(as_estimate(5.0), "estimate")
    expect_equal(x1, estimate(1, 2))
    expect_error(vctrs::vec_c(x1, "a"), class = "vctrs_error_incompatible_type")
})

test_that("math works", {
    x1 = estimate(1, 0.2)
    x2 = estimate(1.5, 0.1)

    expect_equal(x1 + x2, estimate(2.5, sqrt(0.05)))
    expect_equal(x1 - x2, estimate(-0.5, sqrt(0.05)))

    r1 = x1 / x2
    r2 = est_prop(x1, x2)
    expect_gt(field(r1, "se"), field(r2, "se"))

    expect_equal(field(est_pct_chg(x1, x2), "est"), 0.5)

    expect_equal(field(mean(c(x1, x2)), "est"), 1.25)

    r = x1^(0:2)
    expect_equal(r[1], estimate(1, 0))

    r = (0:2)^x1
    expect_equal(r[1:2], estimate(0:1, 0))

    x = estimate(0:5, 0.5)
    r1 = sum(x)
    r2 = mean(x)
    expect_gt(field(r1, "se"), field(r2, "se"))
    expect_equal(field(r2, "est"), 2.5)
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/easycensus-package.R
\docType{package}
\name{easycensus-package}
\alias{easycensus}
\alias{easycensus-package}
\title{easycensus: Quickly Find, Extract, and Marginalize U.S. Census Tables}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Extracting desired data using the proper Census variable names can be time-consuming. This package takes the pain out of that process by providing functions to quickly locate variables and download labeled tables from the Census APIs (\url{https://www.census.gov/data/developers/data-sets.html}).
}
\seealso{
Useful links:
\itemize{
  \item \url{https://corymccartan.com/easycensus/}
  \item \url{https://github.com/CoryMcCartan/easycensus/}
  \item \url{http://corymccartan.com/easycensus/}
  \item Report bugs at \url{https://github.com/CoryMcCartan/easycensus/issues}
}

}
\author{
\strong{Maintainer}: Cory McCartan \email{mccartan@psu.edu}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parse_table.R
\name{cens_parse_tables}
\alias{cens_parse_tables}
\title{Attempt to Parse Tables from a Census API}
\usage{
cens_parse_tables(api, year)
}
\arguments{
\item{api}{A Census API programmatic name such as \code{"acs/acs5"}.}

\item{year}{The year for the data}
}
\value{
A list of \code{cens_table} objects, which are just lists with four elements:
\itemize{
\item \code{concept}, a human-readable name
\item \code{tables}, the constituent table codes
\item \code{surveys}, the supported surveys
\item \code{dims}, the parsed names of the dimensions of the tables
\item \code{vars}, a \code{tibble} with all of the parsed variable values
}
}
\description{
Uses the same parsing code as that which generates \link{tables_sf1} and \link{tables_acs}
See \url{https://www.census.gov/data/developers/data-sets.html} for a list of
APIs and corresponding years, or use \code{\link[censusapi:listCensusApis]{censusapi::listCensusApis()}}.
}
\examples{
\dontrun{
cens_parse_tables("dec/pl", 2020)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimate.R
\name{format.estimate}
\alias{format.estimate}
\title{Format an estimate}
\usage{
\method{format}{estimate}(x, conf = 0.9, digits = 2, trim = FALSE, ..., formatter = fmt_plain)
}
\arguments{
\item{x}{An \link{estimate} vector}

\item{conf}{The confidence level to use in converting the margin of error to
a standard error. Defaults to 90\%, which is what the Census Bureau uses for
ACS estimates.}

\item{digits}{The number of dig}

\item{trim}{logical; if \code{FALSE}, logical, numeric and complex values are
right-justified to a common width: if \code{TRUE} the leading blanks for
justification are suppressed.}

\item{...}{Ignored.}

\item{formatter}{the formatting function to use internally}
}
\description{
Format an estimate for pretty printing
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{cens_margin_to}
\alias{cens_margin_to}
\title{Helper function to sum over nuisance variables}
\usage{
cens_margin_to(data, ...)
}
\arguments{
\item{data}{The output of \code{\link[=cens_get_dec]{cens_get_dec()}} or \code{\link[=cens_get_acs]{cens_get_acs()}}}

\item{...}{The variables of interest, which will be kept. Remaining variables
will be marginalized out.}
}
\value{
A new data frame that has had \code{\link[dplyr:group_by]{dplyr::group_by()}} and
\code{\link[dplyr:summarise]{dplyr::summarize()}} applied.
}
\description{
For ACS data, margins of error will be updated appropriately, using
the functionality in \code{\link[=estimate]{estimate()}}.
}
\examples{
\dontrun{
d_cens = cens_get_acs("state", "B25042")
cens_margin_to(d_cens, bedrooms)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{cens_auth}
\alias{cens_auth}
\title{Authorize use of the Census API}
\usage{
cens_auth()
}
\value{
a Census API key
}
\description{
Tries environment variables \code{CENSUS_API_KEY} and \code{CENSUS_KEY}, in that order.
If none is found and R is used in interactive mode, will prompt the user for
a key.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimate.R
\name{est_prop}
\alias{est_prop}
\alias{est_pct_chg}
\title{Specialized margin-of-error calculations}
\usage{
est_prop(x, y)

est_pct_chg(x, y)
}
\arguments{
\item{x, y}{An \link{estimate} vector. For \code{est_pct_chg()}, calculates the \% change
from \code{x} to \code{y} (i.e., \eqn{(y-x)/x})}
}
\value{
An \link{estimate} vector.
}
\description{
Proportions and percent-change-over-time calculations require different
standard error calculations.
}
\examples{
x = estimate(1, 0.1)
y = estimate(1.5, 0.1)
est_prop(x, y)
est_pct_chg(x, y)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cens_get.R
\name{cens_get_dec}
\alias{cens_get_dec}
\alias{cens_get_acs}
\alias{cens_get_raw}
\alias{cens_get}
\title{Download data from a decennial census or ACS table}
\usage{
cens_get_dec(
  table,
  geo = NULL,
  ...,
  sumfile = "sf1",
  pop_group = NULL,
  check_geo = FALSE,
  drop_total = FALSE,
  show_call = FALSE
)

cens_get_acs(
  table,
  geo = NULL,
  ...,
  year = 2019,
  survey = c("acs5", "acs1"),
  check_geo = FALSE,
  drop_total = FALSE,
  show_call = FALSE
)

cens_get_raw(
  table,
  geo = NULL,
  ...,
  year = 2010,
  api = NULL,
  check_geo = FALSE,
  show_call = TRUE
)
}
\arguments{
\item{table}{The table to download, either as a character vector or a table
object as produced by \code{\link[=cens_find_dec]{cens_find_dec()}},  \code{\link[=cens_find_acs]{cens_find_acs()}} or
\code{\link[=cens_parse_tables]{cens_parse_tables()}}, or as included in \code{tables_dec} and \code{tables_acs}.
Note: some tables are split into A/B/C/etc. versions by race; this function
unifies all of these tables under one code. So, for example, use \code{P012},
not \code{P012A}.}

\item{geo}{The geographic level to return. One of the machine-readable or
human-readable names listed in the "Details" section of \code{\link[=cens_geo]{cens_geo()}}. Will
return all matching geographies of this level, as filtered by the further
arguments to \code{...}.  For example, setting \code{geo="tract"} is equivalent to
setting \code{tract="all"}.}

\item{...}{Geographies to return, as supported by the Census API. Order
matters here---the first argument will be the geographic level to return
(i.e., it corresponds to the \code{geo} argument) and additional arguments will
filter the results. Use \code{"all"}, \code{"*"}, \code{NA}, or \code{TRUE} to return all units
of a particular geography.  See the examples of \code{\link[=cens_geo]{cens_geo()}} for details.}

\item{sumfile}{For decennial data, the summary file to use. SF2 contains more
detailed race and household info.}

\item{pop_group}{For decennial data using summary file SF2, the population
group to filter to. See
\url{https://www2.census.gov/programs-surveys/decennial/2010/technical-documentation/complete-tech-docs/summary-file/sf2.pdf#page=347}.}

\item{check_geo}{If \code{TRUE}, validate the provided geographies against the
available geographies from the relevant Census API.}

\item{drop_total}{Whether to filter out variables which are totals across
another variable. Recommended only after inspection of the underlying
table.}

\item{show_call}{Whether to show the actual call to the Census API. May be
useful for debugging.}

\item{year}{For ACS data, the survey year to get data for.}

\item{survey}{For ACS data, whether to use the one-year or
five-year survey (the default). Make sure to check availability using
\code{\link[=cens_find_acs]{cens_find_acs()}}.}

\item{api}{A Census API programmatic name such as \code{"acs/acs5"}.}
}
\value{
A tibble of census data in tidy format, with columns
\code{GEOID}, \code{NAME}, \code{variable} (containing the Census variable code),
\code{value} or \code{estimate} in the case of ACS tables,
and additional factor columns specific to the table.
}
\description{
Leverages \code{\link[censusapi:getCensus]{censusapi::getCensus()}} to download tables of census data. Tables
are returned in tidy format, with variables given tidy, human-readable names.
}
\section{Functions}{
\itemize{
\item \code{cens_get_dec()}: Get decennial census data.

\item \code{cens_get_acs()}: Get American Community Survey (ACS) data.

\item \code{cens_get_raw()}: Get raw data from another Census Bureau API. Output will
be minimally tidied but will likely require further manipulation.

}}
\examples{
\dontrun{
cens_get_dec("P3", "state")
cens_get_dec(tables_sf1$H2, "state")
cens_get_dec("H2", "county", state="WA", drop_total=TRUE)

cens_get_acs("B09001", county="King", state="WA")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimate.R
\name{estimate}
\alias{estimate}
\alias{is_estimate}
\alias{as_estimate}
\title{Estimate class}
\usage{
estimate(x, se = NULL, moe = NULL, conf = 0.9)

is_estimate(x)

as_estimate(x)
}
\arguments{
\item{x}{A numeric vector containing the estimate(s).}

\item{se}{A numeric vector containing the standard error(s) for the
estimate(s). Users should supply either \code{se} or \code{moe} and \code{conf}.}

\item{moe}{A numeric vector containing the margin(s) of error. Users should
supply either \code{se} or \code{moe} and \code{conf}.}

\item{conf}{The confidence level to use in converting the margin of error to
a standard error. Defaults to 90\%, which is what the Census Bureau uses for
ACS estimates.}
}
\value{
An \code{estimate} vector.
}
\description{
A numeric vector that stores margin-of-error information along with it.
The margin of error will update through basic arithmetic operations, using a
first-order Taylor series approximation. The implicit assumption is that the
errors in each value are uncorrelated. If in fact there is correlation, the
margins of error could be wildly under- or over-estimated.
}
\examples{
estimate(5, 2) # 5 with std. error  2
estimate(15, moe=3) - estimate(5, moe=4)
estimate(1:4, 0.1) * estimate(1, 0.1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parse_table.R
\docType{data}
\name{tables}
\alias{tables}
\alias{tables_sf1}
\alias{tables_acs}
\title{Parsed Census SF1 and ACS Tables}
\format{
A list of \code{cens_table} objects, which are just lists with four elements:
\itemize{
\item \code{concept}, a human-readable name
\item \code{tables}, the constituent table codes
\item \code{surveys}, the supported surveys
\item \code{dims}, the parsed names of the dimensions of the tables
\item \code{vars}, a \code{tibble} with all of the parsed variable values
}

An object of class \code{list} of length 83.

An object of class \code{list} of length 848.
}
\usage{
tables_sf1

tables_acs
}
\description{
Contains parsed table information for the 2010 Decennial Summary File 1 and
2019 ACS 5-year and 1-year tables.
This parsed information is used internally in \code{\link[=cens_find_dec]{cens_find_dec()}},
\code{\link[=cens_find_acs]{cens_find_acs()}}, \code{\link[=cens_get_dec]{cens_get_dec()}}, and \code{\link[=cens_get_acs]{cens_get_acs()}}.
For other sets of tables, try using \code{\link[=cens_parse_tables]{cens_parse_tables()}}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy_labels.R
\name{tidiers}
\alias{tidiers}
\alias{tidy_race}
\alias{tidy_race_detailed}
\alias{tidy_ethnicity}
\alias{tidy_age}
\alias{tidy_age_bins}
\alias{tidy_income_bins}
\alias{tidy_simplify}
\alias{tidy_parens}
\title{Tidy labels in census tables}
\usage{
tidy_race(x)

tidy_race_detailed(x, x2, x3)

tidy_ethnicity(x)

tidy_age(x)

tidy_age_bins(x, as_factor = FALSE)

tidy_income_bins(x, as_factor = FALSE)

tidy_simplify(x)

tidy_parens(x)
}
\arguments{
\item{x}{A factor, which will be re-leveled. Character vectors will be converted to factors.}

\item{x2, x3}{Additional character columns containing detailed information for certain variables (e.g. detailed race)}

\item{as_factor}{if \code{TRUE}, return a factor with levels of the form \verb{[35,40]}.}
}
\value{
A re-leveled factor, except for \code{\link[=tidy_age_bins]{tidy_age_bins()}}, which by default
returns a data frame with columns \code{age_from} and \code{age_to} (inclusive).
}
\description{
Some table labels are quite verbose, and users will often want to shorten them.
These functions make tidying common types of labels easy.
Most produce straightforward output, but there are several more generic tidiers:
\itemize{
\item \code{\link[=tidy_simplify]{tidy_simplify()}} attempts to simplify labels by removing words common to all labels.
\item \code{\link[=tidy_parens]{tidy_parens()}} attempts to simplify labels by removing all terms in parentheses.
\item \code{\link[=tidy_race_detailed]{tidy_race_detailed()}} creates logical columns for each of the six racial categories.
}
}
\examples{
ex_race_long = c("american indian and alaska native alone", "asian alone",
    "black or african american alone", "hispanic or latino",
    "native hawaiian and other pacific islander alone",
    "some other race alone", "total", "two or more races",
    "white alone", "white alone, not hispanic or latino")
tidy_race(ex_race_long)

tidy_age_bins(c("10 to 14 years", "21 years", "85 years and over"))

tidy_parens(c("label one (fake)", "label two (fake)"))
tidy_simplify(c("label one (fake)", "label two (fake)"))

\dontrun{ # requires API key
d = cens_get_acs("B02003", "us", year=2019, survey="acs1")
dplyr::mutate(d, tidy_race_detailed(dtldr_1, dtldr_2, dtldr_3))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cens_find.R
\name{cens_find}
\alias{cens_find}
\alias{cens_find_dec}
\alias{cens_find_acs}
\title{Find a decennial or ACS census table with variables of interest}
\usage{
cens_find(tables, ..., show = 4)

cens_find_dec(..., show = 2)

cens_find_acs(..., show = 4)
}
\arguments{
\item{tables}{A list of \code{cens_table} objects, such as is produced by
\code{\link[=cens_parse_tables]{cens_parse_tables()}}.}

\item{...}{Variables to look for. These can be length-1 character vectors,
or, for convenience, can be left unquoted (see examples).}

\item{show}{How many matching tables to show. Increase this to show more
possible matches, at the cost of more output. Negative values will be
converted to positive but will suppress any printing.}
}
\value{
The codes for the top \code{show} tables, invisibly if \code{show} is positive.
}
\description{
This function uses fuzzy matching to help identify tables from the census
which contain variables of interest. Matched table codes are printed out,
along with the Census-provided table description, the parsed variable names,
and example table cells. The website \url{https://censusreporter.org/} may also
be useful in finding variables.
}
\examples{
cens_find_dec("sex", "age")
cens_find(tables_sf1, "sex", "age") # same as above
cens_find_dec(tenure, race)
cens_find_acs("income", "sex", show=3)
cens_find_acs("heath care", show=-1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimate.R
\name{get_est}
\alias{get_est}
\alias{get_se}
\alias{get_moe}
\alias{to_rvar}
\title{Extract estimates, standard errors, and margins of error}
\usage{
get_est(x)

get_se(x)

get_moe(x, conf = 0.9)

to_rvar(x, n = 500)
}
\arguments{
\item{x}{An \link{estimate} vector.}

\item{conf}{The confidence level to use in constructing the margin of error.}

\item{n}{How many samples to draw.}
}
\value{
An \link{estimate} vector.

A \link[posterior:rvar]{posterior::rvar} vector.
}
\description{
Getter functions for \code{\link[=estimate]{estimate()}} vectors.

The \link[posterior:rvar]{posterior::rvar} class may be useful in handling standard errors for
more complicated mathematical expressions. This function assumes a Normal
distribution centered on the estimate, with standard deviation equal to the
standard error of the estimate. The \code{posterior} package is required for this
function.
}
\examples{
x = estimate(1, 0.1)
get_est(x)
get_moe(x)

x = estimate(1, 0.1)
if (requireNamespace("posterior", quietly=TRUE)) {
    rv_x = to_rvar(x)
    (rv_x^2 / rv_x) - rv_x # std. errors zero (correct)
    x^2 / x - x # std. errors not zero
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cens_geo.R
\name{cens_geo}
\alias{cens_geo}
\title{Construct a Geography Specification for Census Data}
\usage{
cens_geo(geo = NULL, ..., check = TRUE, api = "acs/acs5", year = 2019)
}
\arguments{
\item{geo}{The geographic level to return. One of the machine-readable or
human-readable names listed in the "Details" section. Will return all
matching geographies of this level, as filtered by the further arguments to
\code{...}.  For example, setting \code{geo="tract"} is equivalent to setting
\code{tract="all"}.}

\item{...}{Geographies to return, as supported by the Census API. Order
matters here---the first argument will be the geographic level to return
(i.e., it corresponds to the \code{geo} argument) and additional arguments will
filter the results. Use \code{"all"}, \code{"*"}, \code{NA}, or \code{TRUE} to return all units
of a particular geography.  See the examples for details.}

\item{check}{If \code{TRUE}, validate the provided geographies against the
available geographies from the relevant Census API. Requires the \code{api} and
\code{year} arguments to be specified.}

\item{api}{A Census API programmatic name such as \code{"acs/acs5"}.}

\item{year}{The year for the data}
}
\value{
A list with two elements, \code{region} and \code{regionin}, which together
specify a valid Census API geography argument.
}
\description{
Currently used mostly internally.
Builds a Census API-formatted specification of which geographies to download
data for. State and county names (or postal abbreviations) are partially
matched to existing tables, for ease of use. Other geographies should be
specified with Census GEOIDs. The \code{usgazeteer} package, available with
\code{remotes::install_github("bhaskarvk/usgazetteer")}, may be useful in finding
GEOIDs for other geographies. Consult the "geography" sections of each API
at \url{https://www.census.gov/data/developers/data-sets.html} for information on
which geographic specifiers may be provided in combination with others.
}
\details{
Supported geography arguments:
\itemize{
\item \code{us}
\item \code{region}
\item \code{division}
\item \code{state}
\item \code{county}
\item \code{county_subdiv} (County Subdivision)
\item \code{subminor_civil_division} (Subminor Civil Division)
\item \code{place_remainder} (Place/Remainder (Or Part))
\item \code{tract_part} (Tract (Or Part))
\item \code{urban_rural} (Urban Rural)
\item \code{block_group_part} (Block Group (Or Part))
\item \code{block}
\item \code{tract}
\item \code{aian_area_part} (American Indian Area/Alaska Native Area/Hawaiian Home Land (Or Part))
\item \code{block_group} (Block Group)
\item \code{county_part} (County (Or Part))
\item \code{place_part} (Place (Or Part))
\item \code{place}
\item \code{consolidated_city} (Consolidated City)
\item \code{alaska_native_regional_corporation} (Alaska Native Regional Corporation)
\item \code{aian_area} (American Indian Area/Alaska Native Area/Hawaiian Home Land)
\item \code{tribal_subdiv} (Tribal Subdivision/Remainder)
\item \code{aian_reserve_stat} (American Indian Area/Alaska Native Area (Reservation Or Statistical Entity Only))
\item \code{ai_tribal_subdiv_part} (American Indian Tribal Subdivision (Or Part))
\item \code{ai_off_reserve_trust} (American Indian Area (Off-Reservation Trust Land Only)/Hawaiian Home Land)
\item \code{tribal_census_tract} (Tribal Census Tract)
\item \code{tribal_census_tract_part} (Tribal Census Tract (Or Part))
\item \code{tribal_block_group} (Tribal Block Group)
\item \code{state_part} (State (Or Part))
\item \code{county_subdiv_part} (County Subdivision (Or Part))
\item \code{tribal_subdiv_part} (Tribal Subdivision/Remainder (Or Part))
\item \code{aian_reserve_stat_part} (American Indian Area/Alaska Native Area (Reservation Or Statistical Entity Only) (Or Part))
\item \code{ai_off_reserve_trust_part} (American Indian Area (Off-Reservation Trust Land Only)/Hawaiian Home Land (Or Part))
\item \code{tribal_block_group_part} (Tribal Block Group (Or Part))
\item \code{msa} (Metropolitan Statistical Area/Micropolitan Statistical Area)
\item \code{principal_city_part} (Principal City (Or Part))
\item \code{metro_division} (Metropolitan Division)
\item \code{msa_part} (Metropolitan Statistical Area/Micropolitan Statistical Area (Or Part))
\item \code{metro_division_part} (Metropolitan Division (Or Part))
\item \code{combined_statistical_area} (Combined Statistical Area)
\item \code{combined_necta} (Combined New England City And Town Area)
\item \code{necta} (New England City And Town Area)
\item \code{combined_statistical_area_part} (Combined Statistical Area (Or Part))
\item \code{combined_necta_part} (Combined New England City And Town Area (Or Part))
\item \code{necta_part} (New England City And Town Area (Or Part))
\item \code{principal_city} (Principal City)
\item \code{necta_division} (Necta Division)
\item \code{necta_division_part} (Necta Division (Or Part))
\item \code{urban_area} (Urban Area)
\item \code{urban_area_part} (Urban Area (Or Part))
\item \code{consolidated_city_part} (Consolidated City (Or Part))
\item \code{cd} (Congressional District)
\item \code{sld_upper} (State Legislative District (Upper Chamber))
\item \code{sld_lower} (State Legislative District (Lower Chamber))
\item \code{alaska_native_regional_corporation_part} (Alaska Native Regional Corporation (Or Part))
\item \code{zcta} (Zip Code Tabulation Area)
\item \code{zcta_part} (Zip Code Tabulation Area (Or Part))
\item \code{school_district_elementary} (School District (Elementary))
\item \code{school_district_secondary} (School District (Secondary))
\item \code{school_district_unified} (School District (Unified))
\item \code{congressional_district_part} (Congressional District (Or Part))
\item \code{school_district_elementary_part} (School District (Elementary) (Or Part))
\item \code{school_district_secondary_part} (School District (Secondary) (Or Part))
\item \code{school_district_unified_part} (School District (Unified) (Or Part))
\item \code{voting_district_part} (Voting District (Or Part))
\item \code{subminor_civil_division_part} (Subminor Civil Division (Or Part))
\item \code{state_legislative_district_upper_chamber_part} (State Legislative District (Upper Chamber) (Or Part))
\item \code{state_legislative_district_lower_chamber_part} (State Legislative District (Lower Chamber) (Or Part))
\item \code{vtd} (Voting District)
\item \code{ai_tribal_subdiv} (American Indian Tribal Subdivision)
\item \code{puma} (Public Use Microdata Area)
}
}
\examples{
\dontrun{
cens_geo(state="WA")
cens_geo("county", state="WA") # equivalent to `cens_geo(county="all", state="WA")`
cens_geo(county="King", state="Wash")
cens_geo(zcta="02138", check=FALSE)
cens_geo(zcta=NA, state="WA", check=FALSE)
cens_geo("zcta", state="WA", check=FALSE)
cens_geo(cd="09", state="WA", check=FALSE)
cens_geo("county_part", state="WA", cd="09", check=FALSE)
}

}
